@@ -9,6 +9,7 @@ from __future__ import annotations
 import asyncio
 import base64
 import logging
+import re
 import time
 import uuid
 from collections.abc import Callable
@@ -35,6 +36,18 @@ if TYPE_CHECKING:
 
 _LOGGER = logging.getLogger(__name__)
 
+
+def _mask_area_list(area_list: list[dict[str, Any]]) -> list[dict[str, Any]]:
+    def mask_area(area: dict[str, Any]) -> dict[str, Any]:
+        result = {**area}
+        # Will leave empty names as blank
+        if area.get("name"):
+            result["name"] = "I01BU0tFRF9OQU1FIw=="  # #MASKED_NAME#
+        return result
+
+    return [mask_area(area) for area in area_list]
+
+
 REDACTORS: dict[str, Callable[[Any], Any] | None] = {
     "latitude": lambda x: 0,
     "longitude": lambda x: 0,
@@ -45,15 +58,42 @@ REDACTORS: dict[str, Callable[[Any], Any] | None] = {
     "original_device_id": lambda x: "REDACTED_" + x[9::],  # Strip children
     "nickname": lambda x: "I01BU0tFRF9OQU1FIw==" if x else "",
     "mac": mask_mac,
-    "ssid": lambda x: "I01BU0tFRF9TU0lEIw=" if x else "",
+    "ssid": lambda x: "I01BU0tFRF9TU0lEIw==" if x else "",
     "bssid": lambda _: "000000000000",
+    "channel": lambda _: 0,
     "oem_id": lambda x: "REDACTED_" + x[9::],
-    "setup_code": None,  # matter
-    "setup_payload": None,  # matter
-    "mfi_setup_code": None,  # mfi_ for homekit
-    "mfi_setup_id": None,
-    "mfi_token_token": None,
-    "mfi_token_uuid": None,
+    "hw_id": lambda x: "REDACTED_" + x[9::],
+    "fw_id": lambda x: "REDACTED_" + x[9::],
+    "setup_code": lambda x: re.sub(r"\w", "0", x),  # matter
+    "setup_payload": lambda x: re.sub(r"\w", "0", x),  # matter
+    "mfi_setup_code": lambda x: re.sub(r"\w", "0", x),  # mfi_ for homekit
+    "mfi_setup_id": lambda x: re.sub(r"\w", "0", x),
+    "mfi_token_token": lambda x: re.sub(r"\w", "0", x),
+    "mfi_token_uuid": lambda x: re.sub(r"\w", "0", x),
+    "ip": lambda x: x,  # don't redact but keep listed here for dump_devinfo
+    # smartcam
+    "dev_id": lambda x: "REDACTED_" + x[9::],
+    "ext_addr": lambda x: "REDACTED_" + x[9::],
+    "device_name": lambda x: "#MASKED_NAME#" if x else "",
+    "device_alias": lambda x: "#MASKED_NAME#" if x else "",
+    "alias": lambda x: "#MASKED_NAME#" if x else "",  # child info on parent uses alias
+    "local_ip": lambda x: x,  # don't redact but keep listed here for dump_devinfo
+    # robovac
+    "board_sn": lambda _: "000000000000",
+    "custom_sn": lambda _: "000000000000",
+    "location": lambda x: "#MASKED_NAME#" if x else "",
+    "map_data": lambda x: "#SCRUBBED_MAPDATA#" if x else "",
+    "map_name": lambda x: "I01BU0tFRF9OQU1FIw==",  # #MASKED_NAME#
+    "area_list": _mask_area_list,
+    # unknown robovac binary blob in get_device_info
+    "cd": lambda x: "I01BU0tFRF9CSU5BUlkj",  # #MASKED_BINARY#
+}
+
+# Queries that are known not to work properly when sent as a
+# multiRequest. They will not return the `method` key.
+FORCE_SINGLE_REQUEST = {
+    "getConnectStatus",
+    "scanApList",
 }
 
 
@@ -76,6 +116,7 @@ class SmartProtocol(BaseProtocol):
             self._transport._config.batch_size or self.DEFAULT_MULTI_REQUEST_BATCH_SIZE
         )
         self._redact_data = True
+        self._method_missing_logged = False
 
     def get_smart_request(self, method: str, params: dict | None = None) -> str:
         """Get a request message as a string."""
@@ -157,22 +198,25 @@ class SmartProtocol(BaseProtocol):
         # make mypy happy, this should never be reached..
         raise KasaException("Query reached somehow to unreachable")
 
-    async def _execute_multiple_query(self, requests: dict, retry_count: int) -> dict:
+    async def _execute_multiple_query(
+        self, requests: dict, retry_count: int, iterate_list_pages: bool
+    ) -> dict:
         debug_enabled = _LOGGER.isEnabledFor(logging.DEBUG)
         multi_result: dict[str, Any] = {}
         smart_method = "multipleRequest"
 
-        multi_requests = [
-            {"method": method, "params": params} if params else {"method": method}
-            for method, params in requests.items()
-        ]
-
-        end = len(multi_requests)
+        end = len(requests)
         # The SmartCamProtocol sends requests with a length 1 as a
         # multipleRequest. The SmartProtocol doesn't so will never
         # raise_on_error
         raise_on_error = end == 1
 
+        multi_requests = [
+            {"method": method, "params": params} if params else {"method": method}
+            for method, params in requests.items()
+            if method not in FORCE_SINGLE_REQUEST
+        ]
+
         # Break the requests down as there can be a size limit
         step = self._multi_request_batch_size
         if step == 1:
@@ -192,7 +236,7 @@ class SmartProtocol(BaseProtocol):
 
             smart_params = {"requests": requests_step}
             smart_request = self.get_smart_request(smart_method, smart_params)
-            batch_name = f"multi-request-batch-{batch_num+1}-of-{int(end/step)+1}"
+            batch_name = f"multi-request-batch-{batch_num + 1}-of-{int(end / step) + 1}"
             if debug_enabled:
                 _LOGGER.debug(
                     "%s %s >> %s",
@@ -233,22 +277,41 @@ class SmartProtocol(BaseProtocol):
 
             responses = response_step["result"]["responses"]
             for response in responses:
-                method = response["method"]
+                # some smartcam devices calls do not populate the method key
+                # these should be defined in DO_NOT_SEND_AS_MULTI_REQUEST.
+                if not (method := response.get("method")):
+                    if not self._method_missing_logged:
+                        # Avoid spamming the logs
+                        self._method_missing_logged = True
+                        _LOGGER.error(
+                            "No method key in response for %s, skipping: %s",
+                            self._host,
+                            response_step,
+                        )
+                    # These will end up being queried individually
+                    continue
+
                 self._handle_response_error_code(
                     response, method, raise_on_error=raise_on_error
                 )
                 result = response.get("result", None)
-                await self._handle_response_lists(
-                    result, method, retry_count=retry_count
-                )
+                request_params = rp if (rp := requests.get(method)) else None
+                if iterate_list_pages and result:
+                    await self._handle_response_lists(
+                        result, method, request_params, retry_count=retry_count
+                    )
                 multi_result[method] = result
-        # Multi requests don't continue after errors so requery any missing
+
+        # Multi requests don't continue after errors so requery any missing.
+        # Will also query individually any DO_NOT_SEND_AS_MULTI_REQUEST.
         for method, params in requests.items():
             if method not in multi_result:
                 resp = await self._transport.send(
                     self.get_smart_request(method, params)
                 )
-                self._handle_response_error_code(resp, method, raise_on_error=False)
+                self._handle_response_error_code(
+                    resp, method, raise_on_error=raise_on_error
+                )
                 multi_result[method] = resp.get("result")
         return multi_result
 
@@ -262,7 +325,9 @@ class SmartProtocol(BaseProtocol):
                 smart_method = next(iter(request))
                 smart_params = request[smart_method]
             else:
-                return await self._execute_multiple_query(request, retry_count)
+                return await self._execute_multiple_query(
+                    request, retry_count, iterate_list_pages
+                )
         else:
             smart_method = request
             smart_params = None
@@ -289,12 +354,21 @@ class SmartProtocol(BaseProtocol):
         result = response_data.get("result")
         if iterate_list_pages and result:
             await self._handle_response_lists(
-                result, smart_method, retry_count=retry_count
+                result, smart_method, smart_params, retry_count=retry_count
             )
         return {smart_method: result}
 
+    def _get_list_request(
+        self, method: str, params: dict | None, start_index: int
+    ) -> dict:
+        return {method: {"start_index": start_index}}
+
     async def _handle_response_lists(
-        self, response_result: dict[str, Any], method: str, retry_count: int
+        self,
+        response_result: dict[str, Any],
+        method: str,
+        params: dict | None,
+        retry_count: int,
     ) -> None:
         if (
             response_result is None
@@ -314,8 +388,9 @@ class SmartProtocol(BaseProtocol):
             )
         )
         while (list_length := len(response_result[response_list_name])) < list_sum:
+            request = self._get_list_request(method, params, list_length)
             response = await self._execute_query(
-                {method: {"start_index": list_length}},
+                request,
                 retry_count=retry_count,
                 iterate_list_pages=False,
             )
