@@ -192,6 +192,7 @@ AMBIENT_MODULE = {
 
 
 MOTION_MODULE = {
+    "get_adc_value": {"value": 50, "err_code": 0},
     "get_config": {
         "enable": 0,
         "version": "1.0",
@@ -201,7 +202,7 @@ MOTION_MODULE = {
         "max_adc": 4095,
         "array": [80, 50, 20, 0],
         "err_code": 0,
-    }
+    },
 }
 
 LIGHT_DETAILS = {
@@ -308,10 +309,6 @@ class FakeIotTransport(BaseTransport):
             child_ids = []
         _LOGGER.debug("Setting relay state to %s", x["state"])
 
-        if not child_ids and "children" in self.proto["system"]["get_sysinfo"]:
-            for child in self.proto["system"]["get_sysinfo"]["children"]:
-                child_ids.append(child["id"])
-
         _LOGGER.info("child_ids: %s", child_ids)
         if child_ids:
             for child in self.proto["system"]["get_sysinfo"]["children"]:
