@@ -6,6 +6,7 @@ from typing import Any
 
 from kasa import Credentials, DeviceConfig, SmartProtocol
 from kasa.protocols.smartcamprotocol import SmartCamProtocol
+from kasa.smartcam.smartcamchild import CHILD_INFO_FROM_PARENT, SmartCamChild
 from kasa.transports.basetransport import BaseTransport
 
 from .fakeprotocol_smart import FakeSmartTransport
@@ -33,6 +34,7 @@ class FakeSmartCamTransport(BaseTransport):
         *,
         list_return_size=10,
         is_child=False,
+        get_child_fixtures=True,
         verbatim=False,
         components_not_included=False,
     ):
@@ -52,12 +54,15 @@ class FakeSmartCamTransport(BaseTransport):
         self.verbatim = verbatim
         if not is_child:
             self.info = copy.deepcopy(info)
-            self.child_protocols = FakeSmartTransport._get_child_protocols(
-                self.info, self.fixture_name, "getChildDeviceList"
-            )
+            # We don't need to get the child fixtures if testing things like
+            # lists
+            if get_child_fixtures:
+                self.child_protocols = FakeSmartTransport._get_child_protocols(
+                    self.info, self.fixture_name, "getChildDeviceList", self.verbatim
+                )
         else:
             self.info = info
-        # self.child_protocols = self._get_child_protocols()
+
         self.list_return_size = list_return_size
 
         # Setting this flag allows tests to create dummy transports without
@@ -121,10 +126,26 @@ class FakeSmartCamTransport(BaseTransport):
 
     @staticmethod
     def _get_param_set_value(info: dict, set_keys: list[str], value):
+        cifp = info.get(CHILD_INFO_FROM_PARENT)
+
         for key in set_keys[:-1]:
             info = info[key]
         info[set_keys[-1]] = value
 
+        if (
+            cifp
+            and set_keys[0] == "getDeviceInfo"
+            and (
+                child_info_parent_key
+                := FakeSmartCamTransport.CHILD_INFO_SETTER_MAP.get(set_keys[-1])
+            )
+        ):
+            cifp[child_info_parent_key] = value
+
+    CHILD_INFO_SETTER_MAP = {
+        "device_alias": "alias",
+    }
+
     FIXTURE_MISSING_MAP = {
         "getMatterSetupInfo": (
             "matter",
@@ -132,7 +153,33 @@ class FakeSmartCamTransport(BaseTransport):
                 "setup_code": "00000000000",
                 "setup_payload": "00:0000000-0000.00.000",
             },
-        )
+        ),
+        "getSupportChildDeviceCategory": (
+            "childQuickSetup",
+            {
+                "device_category_list": [
+                    {"category": "ipcamera"},
+                    {"category": "subg.trv"},
+                    {"category": "subg.trigger"},
+                    {"category": "subg.plugswitch"},
+                ]
+            },
+        ),
+        "getScanChildDeviceList": (
+            "childQuickSetup",
+            {
+                "child_device_list": [
+                    {
+                        "device_id": "0000000000000000000000000000000000000000",
+                        "category": "subg.trigger.button",
+                        "device_model": "S200B",
+                        "name": "I01BU0tFRF9OQU1FIw====",
+                    }
+                ],
+                "scan_wait_time": 55,
+                "scan_status": "scanning",
+            },
+        ),
     }
     # Setters for when there's not a simple mapping of setters to getters
     SETTERS = {
@@ -158,15 +205,53 @@ class FakeSmartCamTransport(BaseTransport):
         ],
     }
 
+    def _hub_remove_device(self, info, params):
+        """Remove hub device."""
+        items_to_remove = [dev["device_id"] for dev in params["child_device_list"]]
+        children = info["getChildDeviceList"]["child_device_list"]
+        new_children = [
+            dev for dev in children if dev["device_id"] not in items_to_remove
+        ]
+        info["getChildDeviceList"]["child_device_list"] = new_children
+
+        return {"result": {}, "error_code": 0}
+
     @staticmethod
     def _get_second_key(request_dict: dict[str, Any]) -> str:
-        assert (
-            len(request_dict) == 2
-        ), f"Unexpected dict {request_dict}, should be length 2"
+        assert len(request_dict) == 2, (
+            f"Unexpected dict {request_dict}, should be length 2"
+        )
         it = iter(request_dict)
         next(it, None)
         return next(it)
 
+    def get_child_device_queries(self, method, params):
+        return self._get_method_from_info(method, params)
+
+    def _get_method_from_info(self, method, params):
+        result = copy.deepcopy(self.info[method])
+        if "start_index" in result and "sum" in result:
+            list_key = next(
+                iter([key for key in result if isinstance(result[key], list)])
+            )
+            assert isinstance(params, dict)
+            module_name = next(iter(params))
+
+            start_index = (
+                start_index
+                if (
+                    params
+                    and module_name
+                    and (start_index := params[module_name].get("start_index"))
+                )
+                else 0
+            )
+
+            result[list_key] = result[list_key][
+                start_index : start_index + self.list_return_size
+            ]
+        return {"result": result, "error_code": 0}
+
     async def _send_request(self, request_dict: dict):
         method = request_dict["method"]
 
@@ -191,12 +276,14 @@ class FakeSmartCamTransport(BaseTransport):
                 section = next(iter(val))
                 skey_val = val[section]
                 if not isinstance(skey_val, dict):  # single level query
-                    section_key = section
-                    section_val = skey_val
-                    if (get_info := info.get(get_method)) and section_key in get_info:
-                        get_info[section_key] = section_val
-                    else:
+                    updates = {
+                        k: v for k, v in val.items() if k in info.get(get_method, {})
+                    }
+                    if len(updates) != len(val):
+                        # All keys to update must already be in the getter
                         return {"error_code": -1}
+                    info[get_method] = {**info[get_method], **updates}
+
                     break
                 for skey, sval in skey_val.items():
                     section_key = skey
@@ -221,24 +308,41 @@ class FakeSmartCamTransport(BaseTransport):
                 return {**result, "error_code": 0}
             else:
                 return {"error_code": -1}
+        elif method == "removeChildDeviceList":
+            return self._hub_remove_device(info, request_dict["params"]["childControl"])
+        # actions
+        elif method in [
+            "addScanChildDeviceList",
+            "startScanChildDevice",
+        ]:
+            return {"result": {}, "error_code": 0}
+
+        # smartcam child devices do not make requests for getDeviceInfo as they
+        # get updated from the parent's query. If this is being called from a
+        # child it must be because the fixture has been created directly on the
+        # child device with a dummy parent. In this case return the child info
+        # from parent that's inside the fixture.
+        if (
+            not self.verbatim
+            and method == "getDeviceInfo"
+            and (cifp := info.get(CHILD_INFO_FROM_PARENT))
+        ):
+            mapped = SmartCamChild._map_child_info_from_parent(cifp)
+            result = {"device_info": {"basic_info": mapped}}
+            return {"result": result, "error_code": 0}
+
+        # These methods are handled in get_child_device_query so it can be
+        # patched for tests to simulate dynamic devices.
+        if (
+            method in ("getChildDeviceList", "getChildDeviceComponentList")
+            and method in info
+        ):
+            params = request_dict.get("params")
+            return self.get_child_device_queries(method, params)
 
         if method in info:
             params = request_dict.get("params")
-            result = copy.deepcopy(info[method])
-            if "start_index" in result and "sum" in result:
-                list_key = next(
-                    iter([key for key in result if isinstance(result[key], list)])
-                )
-                start_index = (
-                    start_index
-                    if (params and (start_index := params.get("start_index")))
-                    else 0
-                )
-
-                result[list_key] = result[list_key][
-                    start_index : start_index + self.list_return_size
-                ]
-            return {"result": result, "error_code": 0}
+            return self._get_method_from_info(method, params)
 
         if self.verbatim:
             return {"error_code": -1}
